LIBRARY unicornlib.dll

EXPORTS
  simunicorn_setup_imports
  simunicorn_alloc
  simunicorn_dealloc
  simunicorn_hook
  simunicorn_unhook
  simunicorn_start
  simunicorn_stop
  simunicorn_sync
  simunicorn_bbl_addrs
  simunicorn_stack_pointers
  simunicorn_bbl_addr_count
  simunicorn_syscall_count
  simunicorn_step
  simunicorn_activate_page
  simunicorn_set_last_block_details
  simunicorn_set_stops
  simunicorn_set_map_callback
  simunicorn_cache_page
  simunicorn_uncache_pages_touching_region
  simunicorn_clear_page_cache
  simunicorn_enable_symbolic_reg_tracking
  simunicorn_disable_symbolic_reg_tracking
  simunicorn_symbolic_register_data
  simunicorn_get_symbolic_registers
  simunicorn_is_interrupt_handled
  simunicorn_process_transmit
  simunicorn_set_tracking
  simunicorn_executed_pages
  simunicorn_in_cache
  simunicorn_get_count_of_blocks_with_symbolic_vex_stmts
  simunicorn_get_details_of_blocks_with_symbolic_vex_stmts
  simunicorn_get_stop_details
  simunicorn_set_artificial_registers
  simunicorn_set_cpu_flags_details
  simunicorn_set_register_blacklist
  simunicorn_set_vex_to_unicorn_reg_mappings
  simunicorn_is_interrupt_handled
  simunicorn_set_cgc_syscall_details
  simunicorn_process_transmit
  simunicorn_set_fd_bytes
  simunicorn_set_random_syscall_data
  simunicorn_set_vex_cc_reg_data
  simunicorn_get_count_of_writes_to_reexecute
  simunicorn_get_concrete_writes_to_reexecute
  simunicorn_set_fp_regs_fp_ops_vex_codes
